import re
from typing import Callable, Literal, Optional
from unittest.mock import MagicMock, patch

import pytest
from gmo_fx.api.klines import KlinesApi, KlinesResponse, KlineInterval

from datetime import datetime

from gmo_fx.common import Symbol
from tests.api_test_base import ApiTestBase


class TestKlinesApi(ApiTestBase):

    def create_kline(
        self,
        open_time: str = "0000000000000",
        open: float = 0.0,
        high: float = 0.0,
        low: float = 0.0,
        close: float = 0.0,
    ) -> dict:
        return {
            "openTime": open_time,
            "open": str(open),
            "high": str(high),
            "low": str(low),
            "close": str(close),
        }

    def call_api(
        self,
        symbol: Symbol,
        price_type: Literal["BID", "ASK"],
        interval: KlineInterval,
        date: datetime,
    ) -> KlinesResponse:
        return KlinesApi()(
            symbol,
            price_type,
            interval,
            date,
        )

    def create_klines(
        self, size: int, kline_builder: Optional[Callable[[int], dict]] = None
    ) -> list[dict]:
        kline_builder = kline_builder or (lambda i: self.create_kline())
        return [kline_builder(i) for i in range(size)]

    @patch("gmo_fx.api.api_base.get")
    def test_klines_error(self, get_mock: MagicMock):
        self.check_404_error(
            get_mock,
            lambda: self.call_api(
                Symbol.USD_JPY,
                "BID",
                KlineInterval.Min1,
                datetime(2024, 1, 1),
            ),
            #     get_klines(
            #     Symbol.USD_JPY,
            #     "BID",
            #     KlineInterval.Min1,
            #     datetime(2024, 1, 1),
            # ),
        )

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_klines_from_response_klines_accesser(self, get_mock: MagicMock):
        expect_klines = [
            {
                "openTime": datetime(i + 1980, 1, 1),
                "open": float(i),
                "high": float(i),
                "low": float(i),
                "close": float(i),
            }
            for i in range(100)
        ]
        get_mock.return_value = self.create_response(
            data=self.create_klines(
                100,
                lambda i: self.create_kline(
                    open_time=str(int(expect_klines[i]["openTime"].timestamp()) * 1000),
                    open=expect_klines[i]["open"],
                    high=expect_klines[i]["high"],
                    low=expect_klines[i]["low"],
                    close=expect_klines[i]["close"],
                ),
            )
        )
        klines_response = self.call_api(
            Symbol.USD_JPY,
            "BID",
            KlineInterval.Min1,
            datetime(2024, 1, 1),
        )
        for i, expect in enumerate(expect_klines):
            assert klines_response.klines[i].open_time == expect["openTime"]
            assert klines_response.klines[i].open == expect["open"]
            assert klines_response.klines[i].high == expect["high"]
            assert klines_response.klines[i].low == expect["low"]
            assert klines_response.klines[i].close == expect["close"]

    def check_url_parameter(
        self,
        get_mock: MagicMock,
        symbol: Symbol = Symbol.USD_JPY,
        symbol_str: str = "USD_JPY",
        price_type: str = "ASK",
        price_type_str: str = "ASK",
        interval=KlineInterval.Min1,
        interval_str="1min",
        date: datetime = datetime(2024, 1, 1),
        date_str: str = "20240101",
    ) -> None:

        get_mock.return_value = self.create_response(data=self.create_klines(1))
        self.call_api(symbol, price_type, interval, date)
        param_match = re.search("\?(.*)", get_mock.mock_calls[0].args[0])
        param = param_match.group(1)
        assert f"symbol={symbol_str}" in param
        assert f"priceType={price_type_str}" in param
        assert f"interval={interval_str}" in param
        assert f"date={date_str}" in param

    symbol_strs = [
        (Symbol.USD_JPY, "USD_JPY"),
        (Symbol.EUR_JPY, "EUR_JPY"),
        (Symbol.GBP_JPY, "GBP_JPY"),
        (Symbol.AUD_JPY, "AUD_JPY"),
        (Symbol.NZD_JPY, "NZD_JPY"),
        (Symbol.CAD_JPY, "CAD_JPY"),
        (Symbol.CHF_JPY, "CHF_JPY"),
        (Symbol.TRY_JPY, "TRY_JPY"),
        (Symbol.ZAR_JPY, "ZAR_JPY"),
        (Symbol.MXN_JPY, "MXN_JPY"),
        (Symbol.EUR_USD, "EUR_USD"),
        (Symbol.GBP_USD, "GBP_USD"),
        (Symbol.AUD_USD, "AUD_USD"),
        (Symbol.NZD_USD, "NZD_USD"),
    ]

    @pytest.mark.parametrize("symbol, symbol_str", symbol_strs)
    @patch("gmo_fx.api.api_base.get")
    def test_should_call_get_with_symbol(
        self, get_mock: MagicMock, symbol: Symbol, symbol_str: str
    ):
        self.check_url_parameter(
            get_mock,
            symbol=symbol,
            symbol_str=symbol_str,
        )

    price_type_strs = [
        ("BID", "BID"),
        ("ASK", "ASK"),
    ]

    @pytest.mark.parametrize("price_type, price_type_str", price_type_strs)
    @patch("gmo_fx.api.api_base.get")
    def test_should_call_get_with_price_type(
        self, get_mock: MagicMock, price_type: str, price_type_str: str
    ):
        self.check_url_parameter(
            get_mock,
            price_type=price_type,
            price_type_str=price_type_str,
        )

    interval_strs = [
        (KlineInterval.Min1, "1min"),
        (KlineInterval.Min5, "5min"),
        (KlineInterval.Min10, "10min"),
        (KlineInterval.Min15, "15min"),
        (KlineInterval.Min30, "30min"),
        (KlineInterval.H1, "1hour"),
        (KlineInterval.H4, "4hour"),
        (KlineInterval.H8, "8hour"),
        (KlineInterval.H12, "12hour"),
        (KlineInterval.D1, "1day"),
        (KlineInterval.W1, "1week"),
        (KlineInterval.M1, "1month"),
    ]

    date_strs = [
        # datetime, interval, string
        (datetime(2024, 1, 1, 10, 23, 12), KlineInterval.Min1, "20240101"),
        (datetime(2024, 1, 1, 10, 23, 12), KlineInterval.Min5, "20240101"),
        (datetime(2024, 1, 1, 10, 23, 12), KlineInterval.Min10, "20240101"),
        (datetime(2024, 1, 1, 10, 23, 12), KlineInterval.Min15, "20240101"),
        (datetime(2024, 1, 1, 10, 23, 12), KlineInterval.Min30, "20240101"),
        (datetime(2024, 1, 1, 10, 23, 12), KlineInterval.H1, "20240101"),
        (datetime(2024, 1, 1, 10, 23, 12), KlineInterval.H4, "2024"),
        (datetime(2024, 1, 1, 10, 23, 12), KlineInterval.H8, "2024"),
        (datetime(2024, 1, 1, 10, 23, 12), KlineInterval.H12, "2024"),
        (datetime(2024, 1, 1, 10, 23, 12), KlineInterval.D1, "2024"),
        (datetime(2024, 1, 1, 10, 23, 12), KlineInterval.W1, "2024"),
        (datetime(2024, 1, 1, 10, 23, 12), KlineInterval.M1, "2024"),
    ]

    @pytest.mark.parametrize("date, interval, string", date_strs)
    @patch("gmo_fx.api.api_base.get")
    def test_should_call_get_with_date(
        self, get_mock: MagicMock, date: datetime, interval: KlineInterval, string: str
    ):
        interval_map = dict(self.interval_strs)
        self.check_url_parameter(
            get_mock,
            interval=interval,
            interval_str=interval_map[interval],
            date=date,
            date_str=string,
        )

    @patch("gmo_fx.api.api_base.get")
    def test_check_url(
        self,
        get_mock: MagicMock,
    ) -> None:
        get_mock.return_value = self.create_response(data=self.create_klines(1))
        self.call_api(Symbol.USD_JPY, "BID", KlineInterval.D1, datetime.now())
        url_match = re.search("(.*)\?.*", get_mock.mock_calls[0].args[0])
        url = url_match.group(1)
        assert url == "https://forex-api.coin.z.com/public/v1/klines"



import re
from typing import Callable, Optional
from unittest.mock import MagicMock, patch
from gmo_fx.common import Symbol
from gmo_fx.api.position_summary import (
    Position,
    PositionSummaryApi,
    PositionSummaryResponse,
)

from tests.api_test_base import ApiTestBase


class TestPositionSummaryApi(ApiTestBase):

    def call_api(
        self,
        symbol: Symbol = Symbol.USD_JPY,
    ) -> PositionSummaryResponse:
        return PositionSummaryApi(
            api_key="",
            secret_key="",
        )(symbol=symbol)

    def create_position_summary_data(
        self,
        average_position_rate: float = 0.0,
        position_loss_gain: float = 0.0,
        side: str = "BUY",
        sum_ordered_size: int = 0,
        sum_position_size: int = 0,
        sum_total_swap: float = 0.0,
        symbol: str = "USD_JPY",
    ) -> dict:
        return {
            "averagePositionRate": average_position_rate,
            "positionLossGain": position_loss_gain,
            "side": side,
            "sumOrderedSize": sum_ordered_size,
            "sumPositionSize": sum_position_size,
            "sumTotalSwap": sum_total_swap,
            "symbol": symbol,
        }

    @patch("gmo_fx.api.api_base.get")
    def test_404_error(self, get_mock: MagicMock):
        self.check_404_error(get_mock, lambda: self.call_api())

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_average_position_rate(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[self.create_position_summary_data(average_position_rate=1.0)]
        )
        response = self.call_api()
        average_position_rates = [
            position.average_position_rate for position in response.positions
        ]
        assert average_position_rates[0] == 1.0

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_position_loss_gain(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[self.create_position_summary_data(position_loss_gain=1.0)]
        )
        response = self.call_api()
        position_loss_gains = [
            position.position_loss_gain for position in response.positions
        ]
        assert position_loss_gains[0] == 1.0

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_position_side_buy(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[self.create_position_summary_data(side="BUY")]
        )
        response = self.call_api()
        sides = [position.side for position in response.positions]
        assert sides[0] == Position.Side.BUY

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_position_side_sell(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[self.create_position_summary_data(side="SELL")]
        )
        response = self.call_api()
        sides = [position.side for position in response.positions]
        assert sides[0] == Position.Side.SELL

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_position_sum_ordered_size(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[self.create_position_summary_data(sum_ordered_size=984)]
        )
        response = self.call_api()
        sum_ordered_sizes = [
            position.sum_ordered_size for position in response.positions
        ]
        assert sum_ordered_sizes[0] == 984

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_position_sum_position_size(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[self.create_position_summary_data(sum_position_size=984)]
        )
        response = self.call_api()
        sum_position_sizes = [
            position.sum_position_size for position in response.positions
        ]
        assert sum_position_sizes[0] == 984

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_position_sum_total_swap(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[self.create_position_summary_data(sum_total_swap=654.6)]
        )
        response = self.call_api()
        sum_total_swaps = [position.sum_total_swap for position in response.positions]
        assert sum_total_swaps[0] == 654.6

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_position_symbol_usd_jpy(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[self.create_position_summary_data(symbol="USD_JPY")]
        )
        response = self.call_api()
        symbols = [position.symbol for position in response.positions]
        assert symbols[0] == Symbol.USD_JPY

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_position_symbol_gbp_usd(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[self.create_position_summary_data(symbol="GBP_USD")]
        )
        response = self.call_api()
        symbols = [position.symbol for position in response.positions]
        assert symbols[0] == Symbol.GBP_USD

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_some_positions(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[
                self.create_position_summary_data(symbol="USD_JPY"),
                self.create_position_summary_data(symbol="GBP_USD"),
            ]
        )
        response = self.call_api()
        symbols = [position.symbol for position in response.positions]
        assert symbols[0] == Symbol.USD_JPY
        assert symbols[1] == Symbol.GBP_USD

    @patch("gmo_fx.api.api_base.get")
    def test_check_url(
        self,
        get_mock: MagicMock,
    ) -> None:
        get_mock.return_value = self.create_response(data=[])
        self.call_api()
        url = get_mock.mock_calls[0].args[0]
        assert (
            url
            == "https://forex-api.coin.z.com/private/v1/positionSummary?symbol=USD_JPY"
        )

    @patch("gmo_fx.api.api_base.get")
    def test_should_call_api_with_symbol(
        self,
        get_mock: MagicMock,
    ) -> None:
        get_mock.return_value = self.create_response(data=[])
        self.call_api(symbol=Symbol.AUD_JPY)
        url = get_mock.mock_calls[0].args[0]
        assert "symbol=AUD_JPY" in url



from datetime import datetime, timedelta, timezone
import re
from typing import Callable, Optional
from unittest.mock import MagicMock, patch
from gmo_fx.common import SettleType, Side, Symbol
from gmo_fx.api.latest_executions import (
    Execution,
    LatestExecutionsApi,
    LatestExecutionsResponse,
)

from tests.api_test_base import ApiTestBase


class TestLatestExecutionsApi(ApiTestBase):

    def call_api(
        self,
        symbol: Symbol = Symbol.USD_JPY,
        count: int = 100,
    ) -> LatestExecutionsResponse:
        return LatestExecutionsApi(
            api_key="",
            secret_key="",
        )(symbol, count)

    def create_execution_data(
        self,
        amount: float = 0.0,
        execution_id: int = 0,
        client_order_id: str = "id",
        order_id: int = 0,
        position_id: int = 0,
        symbol: str = "USD_JPY",
        side: str = "SELL",
        settle_type: str = "CLOSE",
        size: int = 1,
        price: float = 0.0,
        loss_gain: int = 0,
        fee: int = 0,
        settled_swap: float = 0.0,
        timestamp: datetime = datetime.now(),
    ) -> dict:
        return {
            "amount": amount,
            "executionId": execution_id,
            "clientOrderId": client_order_id,
            "orderId": order_id,
            "positionId": position_id,
            "symbol": symbol,
            "side": side,
            "settleType": settle_type,
            "size": f"{size}",
            "price": f"{price}",
            "lossGain": f"{loss_gain}",
            "fee": f"{fee}",
            "settledSwap": f"{settled_swap}",
            "timestamp": timestamp.astimezone(timezone.utc).strftime(
                "%Y-%m-%dT%H:%M:%S.%fZ"
            ),
        }

    @patch("gmo_fx.api.api_base.get")
    def test_404_error(self, get_mock: MagicMock):
        self.check_404_error(get_mock, lambda: self.call_api())

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_amount(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[self.create_execution_data(amount=1.2)]
        )
        response = self.call_api()
        amounts = [execution.amount for execution in response.executions]
        assert amounts[0] == 1.2

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_execution_id(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[self.create_execution_data(execution_id=12)]
        )
        response = self.call_api()
        execution_ids = [execution.execution_id for execution in response.executions]
        assert execution_ids[0] == 12

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_client_order_id(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[self.create_execution_data(client_order_id="fdsa")]
        )
        response = self.call_api()
        client_order_ids = [
            execution.client_order_id for execution in response.executions
        ]
        assert client_order_ids[0] == "fdsa"

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_order_id(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[self.create_execution_data(order_id=12)]
        )
        response = self.call_api()
        order_ids = [execution.order_id for execution in response.executions]
        assert order_ids[0] == 12

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_position_id(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[self.create_execution_data(position_id=12)]
        )
        response = self.call_api()
        position_ids = [execution.position_id for execution in response.executions]
        assert position_ids[0] == 12

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_symbol(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[self.create_execution_data(symbol="NZD_USD")]
        )
        response = self.call_api()
        symbols = [execution.symbol for execution in response.executions]
        assert symbols[0] == Symbol.NZD_USD

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_side_buy(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[self.create_execution_data(side="BUY")]
        )
        response = self.call_api()
        sides = [execution.side for execution in response.executions]
        assert sides[0] == Side.BUY

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_side_sell(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[self.create_execution_data(side="SELL")]
        )
        response = self.call_api()
        sides = [execution.side for execution in response.executions]
        assert sides[0] == Side.SELL

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_settle_type_open(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[self.create_execution_data(settle_type="OPEN")]
        )
        response = self.call_api()
        settle_types = [execution.settle_type for execution in response.executions]
        assert settle_types[0] == SettleType.OPEN

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_settle_type_close(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[self.create_execution_data(settle_type="CLOSE")]
        )
        response = self.call_api()
        settle_types = [execution.settle_type for execution in response.executions]
        assert settle_types[0] == SettleType.CLOSE

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_size(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[self.create_execution_data(size=100)]
        )
        response = self.call_api()
        sizes = [execution.size for execution in response.executions]
        assert sizes[0] == 100

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_price(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[self.create_execution_data(price=100.5)]
        )
        response = self.call_api()
        prices = [execution.price for execution in response.executions]
        assert prices[0] == 100.5

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_loss_gain(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[self.create_execution_data(loss_gain=30)]
        )
        response = self.call_api()
        loss_gains = [execution.loss_gain for execution in response.executions]
        assert loss_gains[0] == 30

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_fee(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[self.create_execution_data(fee=40)]
        )
        response = self.call_api()
        fees = [execution.fee for execution in response.executions]
        assert fees[0] == 40

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_settled_swap(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[self.create_execution_data(settled_swap=40.4)]
        )
        response = self.call_api()
        settled_swaps = [execution.settled_swap for execution in response.executions]
        assert settled_swaps[0] == 40.4

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_timestamp(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[
                self.create_execution_data(
                    timestamp=datetime(2024, 1, 2, 20, 11, tzinfo=timezone.utc)
                )
            ]
        )
        response = self.call_api()
        timestamps = [execution.timestamp for execution in response.executions]
        delta = datetime(2024, 1, 2, 20, 11, tzinfo=timezone.utc) - timestamps[0]
        assert delta.seconds == 0

    @patch("gmo_fx.api.api_base.get")
    def test_should_set_url_symbol(self, get_mock: MagicMock):

        get_mock.return_value = self.create_response(
            data=[self.create_execution_data()]
        )
        response = self.call_api(symbol=Symbol.EUR_USD)

        param_match = re.search("\?(.*)", get_mock.mock_calls[0].args[0])
        param = param_match.group(1)
        assert f"symbol=EUR_USD" in param

    @patch("gmo_fx.api.api_base.get")
    def test_should_set_url_count(self, get_mock: MagicMock):

        get_mock.return_value = self.create_response(
            data=[self.create_execution_data()]
        )
        response = self.call_api(count=20)

        param_match = re.search("\?(.*)", get_mock.mock_calls[0].args[0])
        param = param_match.group(1)
        assert f"count=20" in param

    @patch("gmo_fx.api.api_base.get")
    def test_check_url(
        self,
        get_mock: MagicMock,
    ) -> None:
        get_mock.return_value = self.create_response(data=[])
        self.call_api()
        url_match = re.search("(.*)\?.*", get_mock.mock_calls[0].args[0])
        url = url_match.group(1)
        assert url == "https://forex-api.coin.z.com/private/v1/latestExecutions"
